# -*- coding: utf-8 -*-
#
# Copyright © 2015 Chaoyi Zha <me@cydrobolt.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import requests, json
import util

class PIFX:
    def __init__(self, api_key, http_endpoint=None):
        # use default API base endpoint if none defined
        if http_endpoint == None:
            self.http_base = "https://api.lifx.com/v1/"
        else:
            self.http_base = http_endpoint

        self.api_key = api_key
        # generate HTTP authentication header
        self.headers = util.generate_auth_header(self.api_key)
        self._s = requests.Session()

    def full_http_endpoint(self, suffix):
        return self.http_base + suffix

    def list_lights(self, selector='all'):
        """
        Given a selector (defaults to all), return a list of lights.
        Without a selector provided, return list of all lights.
        """

        endpoint = self.full_http_endpoint(
            "lights/{}".format(selector)
        )
        res = self._s.get(endpoint, headers=self.headers)
        parsed_data = util.parse_response(res)

        util.handle_error(res)

        return parsed_data

    def set_state(self, selector='all',
        power=None, color=None, brightness=None, duration=None):
        """
        Given a selector (defaults to all), set the state of a light.
        Selector can be based on id, scene_id, group_id, label, etc.
        Returns list of lightbulb statuses if successful.
        See http://api.developer.lifx.com/v1/docs/selectors

        selector: required String
            The selector to limit which lights will run the effect.

        power: String
            e.g "on" or "off"

        color: String
            e.g #ff0000 or "red"
            Color to set selected bulbs.
            Hex color code, color name, saturation percentage, hue, RGB, etc.
            See http://api.developer.lifx.com/v1/docs/colors

        brightness: Double
            e.g 0.5
            Set brightness level from 0 to 1

        duration: Double
            e.g 10
            Setting transition time, in seconds, from 0.0 to 3155760000.0 (100 years).

        """

        endpoint = self.full_http_endpoint(
            "lights/{}/state".format(selector)
        )
        argument_tuples = [
            ('power', power),
            ('color', color),
            ('brightness', brightness),
            ('duration', duration)
        ]
        data = util.arg_tup_to_dict(argument_tuples)

        res = self._s.put(endpoint, data=data, headers=self.headers)
        parsed_data = util.parse_response(res)

        util.handle_error(res)

        return util.parse_data(parsed_data)

    def toggle_power(self, selector='all', duration=1.0):
        """Given a selector and transition duration, toggle lights (on/off)"""
        endpoint = self.full_http_endpoint(
            "lights/{}/toggle".format(selector)
        )

        data = {
            "duration": duration
        }

        res = self._s.post(endpoint, data=data, headers=self.headers)
        parsed_data = util.parse_response(res)

        util.handle_error(res)

        return util.parse_data(parsed_data)

    def breathe_lights(self, color, selector='all',
        from_color=None, period=1.0, cycles=1.0,
        persist=False, power_on=True, peak=0.5, duration=1.0):
        """
        Perform breathe effect on lights.

        selector: String
            The selector to limit which lights will run the effect.
            default: all

        color: required String
            Color attributes to use during effect. See set_state for more.

        from_color:	String
            The color to start the effect from. See set_state for more.
            default: current bulb color

        period:	Double
            The time in seconds for one cyles of the effect.
            default: 1.0

        cycles:	Double
            The number of times to repeat the effect.
            default: 1.0

        persist: Boolean
            If false set the light back to its previous
            value when effect ends, if true leave the last effect color.
            default: false

        power_on: Boolean
            If true, turn the bulb on if it is not already on.
            default: true

        peak: String
            Defines where in a period the target color is at its maximum.
            Minimum 0.0, maximum 1.0.
            default: 0.5

        """
        endpoint = self.full_http_endpoint(
            "lights/{}/effects/breathe".format(selector)
        )

        data = {
            "selector": selector,
            "color": color,
            "from_color": from_color,
            "period": period,
            "cycles": cycles,
            "persist": persist,
            "power_on": power_on,
            "peak": peak,
            "duration": duration
        }

        res = self._s.post(endpoint, data=data, headers=self.headers)
        parsed_data = util.parse_response(res)

        util.handle_error(res)

        return util.parse_data(parsed_data)

    def pulse_lights(self, color, selector='all',
        from_color=None, period=1.0, cycles=1.0,
        persist=False, power_on=True, peak=0.5, duration=1.0):
        """
        Perform pulse effect on lights.

        selector: String
            The selector to limit which lights will run the effect.
            default: all

        color: required String
            Color attributes to use during effect. See set_state for more.

        from_color:	String
            The color to start the effect from. See set_state for more.
            default: current bulb color

        period:	Double
            The time in seconds for one cyles of the effect.
            default: 1.0

        cycles:	Double
            The number of times to repeat the effect.
            default: 1.0

        persist: Boolean
            If false set the light back to its previous
            value when effect ends, if true leave the last effect color.
            default: false

        power_on: Boolean
            If true, turn the bulb on if it is not already on.
            default: true

        peak: String
            Defines where in a period the target color is at its maximum.
            Minimum 0.0, maximum 1.0.
            default: 0.5

        """
        endpoint = self.full_http_endpoint(
            "lights/{}/effects/pulse".format(selector)
        )

        data = {
            "color": color,
            "from_color": from_color,
            "period": period,
            "cycles": cycles,
            "persist": persist,
            "power_on": power_on,
            "peak": peak,
            "duration": duration
        }

        res = self._s.post(endpoint, data=data, headers=self.headers)
        parsed_data = util.parse_response(res)

        util.handle_error(res)

        return util.parse_data(parsed_data)

    def cycle_lights(self, states,
        defaults, direction='forward', selector='all',):
        """
        Cycle through list of effects.

        Provide array states as a list of dictionaries with set_state arguments.
        See http://api.developer.lifx.com/docs/cycle

        selector: String
            The selector to limit which lights will run the effect.
            default: all

        states: required List of Dicts
            List of arguments, named as per set_state. Must have 2 to 5 entries.

        defaults: Object
            Default values to use when not specified in each states[] object.
            Argument names as per set_state.

        direction: String
            Direction in which to cycle through the list. Can be forward or backward
            default: forward
        """
        endpoint = self.full_http_endpoint(
            "lights/{}/cycle".format(selector)
        )

        states_json = json.dumps(states)

        data = {
            'states': states_json,
            'defaults': defaults,
            'direction': direction
        }

        res = self._s.post(endpoint, data=data, headers=self.headers)
        parsed_data = util.parse_response(res)

        util.handle_error(res)

        return util.parse_data(parsed_data)

    def list_scenes(self):
        """
        Return a list of scenes.
        See http://api.developer.lifx.com/docs/list-scenes
        """

        endpoint = self.full_http_endpoint(
            "scenes"
        )
        res = self._s.get(endpoint, headers=self.headers)
        parsed_data = util.parse_response(res)

        util.handle_error(res)

        return parsed_data

    def activate_scene(self, scene_uuid, duration=1.0):
        """
        Return a list of scenes.

        See http://api.developer.lifx.com/docs/activate-scene

        scene_uuid: required String
            The UUID for the scene you wish to activate

        duration: Double
            The time in seconds to spend performing the scene transition.
            default: 1.0
        """

        endpoint = self.full_http_endpoint(
            "scenes/scene_id:{}/activate".format(scene_uuid)
        )

        data = {
            'duration': duration
        }

        res = self._s.put(endpoint, data=data, headers=self.headers)
        parsed_data = util.parse_response(res)

        return parsed_data



# -*- coding: utf-8 -*-
#
# Copyright © 2015 Chaoyi Zha <me@cydrobolt.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
from os import path

here = path.abspath(path.dirname(__file__))

setup(
    name='pifx',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version='0.0.1',

    description="""PIFX is a Python library for the LIFX cloud HTTP API""",

    # The project's main homepage.
    url='https://github.com/cydrobolt/pifx',

    # Author details
    author='Chaoyi Zha',
    author_email='me@cydrobolt.com',
    # License
    license='Apache v2',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',

        'License :: OSI Approved :: Apache Software License',
        'Topic :: Software Development :: Libraries'
    ],

    keywords='lifx iot library lights sdk',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=[
        'requests'
    ],

    include_package_data=True,

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    entry_points={
        # 'console_scripts': [
        #
        # ],
    },
)



# -*- coding: utf-8 -*-
#
# Copyright © 2015 Chaoyi Zha <me@cydrobolt.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

A_OK_HTTP_CODES = [
    200,
    207
]

A_ERROR_HTTP_CODES = {
    400: "Request was invalid",
    401: "Invalid API key",
    403: "Bad OAuth scope",
    404: "Selector did not match any lights",
    422: "Missing or malformed parameters",
    426: "HTTP is required to perform transaction",
    # see http://api.developer.lifx.com/v1/docs/rate-limits
    429: "Rate limit exceeded",
    500: "API currently unavailable",
    502: "API currently unavailable",
    503: "API currently unavailable",
    523: "API currently unavailable"
}
